; --------------------------------------------------------------------------------------------------------------------------------
; Loading section

(lib-load "xml-in-laml/xml-in-laml.scm")
(lib-load "time.scm")

(define (song-front-matters! ast)
  (do-song-front-matters ast))

(define (song-verses! ast)
  (do-song-verses ast))

(set! xml-check-language-overlap? #f)


(lib-load "xml-in-laml/mirrors/xhtml10-transitional-mirror.scm")
(define xhtml:em (xhtml10-transitional 'em))
(lib-load "xhtml1.0-convenience.scm")

; Loading chords XML-in-LAML stuff. 
; Overwrites a and em from the XHTML mirror.
(laml-style "xml-in-laml/chords/mirror/chords")

; Accept non-strings as attribute values in songs with chords:
(set-xml-accept-only-string-valued-attributes-in 'chords #f)

; ---------------------------------------------------------------------------------------------------------------
; Some global variables, used for non-local communication.

(define the-song-title "")
(define the-song-author "")
(define the-font-size 100)
(define the-indentation 0)
(define the-chord-color "black")
(define compact-lines-wanted #f)
(define do-show-chords #t)

; ---------------------------------------------------------------------------------------------------------------

(define (do-song-front-matters ast)
  (let* ((song-title (find-first-ast ast "song-title")) 
         (title-str (if song-title (ast-text song-title) #f))
         (song-author (find-first-ast ast "song-author"))
         (author-str (if song-author (ast-text song-author) #f)) 

         (font-size (as-number (ast-attribute ast 'font-size 100)))
         (indentation (as-number (ast-attribute ast 'indentation 0)))
         (transposition-number (as-number (ast-attribute ast 'transposition 0)))
         (compact-lines? (as-boolean (ast-attribute ast 'compact-lines "false")))
         (chord-color (ast-attribute ast 'chord-color "black"))
        )

   (set! the-song-title title-str)
   (set! the-song-author author-str)
   (set! the-font-size font-size)
   (set! the-indentation indentation)
   (set! transposition transposition-number)  ; a global variable
   (set! compact-lines-wanted compact-lines?)
   (set! the-chord-color chord-color)
))

(define (do-song-verses ast)
 (let ((xhtml:title (xhtml10-transitional 'title))
       (verses (find-asts ast "verse" verse-transform)))
   (write-html '(pp prolog)
     (html
      (head (xhtml:title the-song-title))
      (body 
       (cond ((and the-song-title the-song-author) 
               (left-right-banner "" (div 'css:text-align "right"  
                                       (span 'css:font-size (string-append (as-string the-font-size) "%")
                                             (b the-song-title) (br) (xhtml:em the-song-author))  (p))))
             (the-song-title 
               (left-right-banner "" (div 'css:text-align "right"  
                                       (span 'css:font-size (string-append (as-string the-font-size) "%")
                                             (b the-song-title)) (p))))
             (the-song-author
               (left-right-banner "" (div 'css:text-align "right"  
                                      (span 'css:font-size (string-append (as-string the-font-size) "%")
                                             (xhtml:em the-song-author)) (p))))
             (else '()))

       (indent-pixels 
         the-indentation
         (div 'css:font-size (string-append (as-string the-font-size) "%")
           verses))
 
       (left-right-banner "" (font 'size "1" 'color "grey" (when-generated)))

       ))
      (in-startup-directory (string-append (source-filename-without-extension) "." "html")))))


(define (verse-transform verse-ast)
 (let ((show-chords? (as-boolean (ast-attribute verse-ast 'chords "true"))))
  (set! do-show-chords show-chords?)
  (let ((lines (find-asts verse-ast "line" line-transform))
        (n (ast-attribute verse-ast 'n #f))
       )
    (div (if show-chords? (list 'css:line-height "7ex") '())
         (if n (span 'css:margin-left "-3ex" (b (string-append (as-string n) ".")) (horizontal-space 1)) '()) 
         lines (p)))))

(define (line-transform line-ast)
 (span 
  (xhtml:em 
   (transform-ast-list
    (ast-subtrees line-ast)
    (list chord-ast? chord-transform)) 
   (br))))

(define (chord-ast? x)
  (and (ast? x)
       (member (ast-element-name x) (list "c"  "cs"  "d"  "eb"  "e"  "f"  "fs"  "g"  "gs"  "a"  "hb"  "h"))))

(define (chord-transform chord-ast)
  (let* ((phrase (ast-text chord-ast))
         (phrase1 (if (blank-string? phrase) #f phrase))
         (abstr (ast-attribute chord-ast 'x))
        )
    (phrase-with-chord phrase1 (as-symbol (ast-element-name chord-ast)) abstr))) 

;---------------------------------------------------------------------------------------------------------------
; Transpose stuff:

(define wheel '(c cs d eb e f fs g gs a hb h))

(define minus-wheel (reverse wheel))

; Transponer song et antal steps. 
; Et positiv steps vrdi transponerer med uret p hjule.
; En negativ steps vrdi transponener mod uret.
; song er en liste af noder. En node kan vre et symbol, eller en liste af 
; det primre navn og dets abstraktion: c eller (c) eller (c 7).
(define (transpose-song song steps)
  (map (transpose-node steps) song))

(define (transpose-node steps)
  (lambda (node) 
    (cond ((and (symbol? node) (or (eq? node 'x) (eq? node 'X))) 'X) 
          ((symbol? node) (transpose-primary-node node steps))
          ((list? node) (cons (transpose-primary-node (car node) steps) (cdr node)))
          (else (laml-error "transpose-node: Unknown node")))))

; Steps is in between -11 and 11
(define (transpose-primary-node node steps)
 (if (>= steps 0)
     (let* ((node-val (index-in-list-by-predicate wheel node eq?))
            (transposed-node-val (+ node-val steps))
            (transposed-node (cond ((and (>= transposed-node-val 0) (<= transposed-node-val 11)) 
                                      (list-ref wheel transposed-node-val))
                                   ((and (>= transposed-node-val 12) (<= transposed-node-val 23))
                                    (list-ref wheel (- transposed-node-val 12)))
                                   (else (laml-error "tranpose-primary-node: Problems (1)")))))
        transposed-node)
     (let* ((node-val (index-in-list-by-predicate minus-wheel node eq?))
            (transposed-node-val (+ node-val (- steps)))
            (transposed-node (cond ((and (>= transposed-node-val 0) (<= transposed-node-val 11)) 
                                      (list-ref minus-wheel transposed-node-val))
                                   ((and (>= transposed-node-val 12) (<= transposed-node-val 23))
                                    (list-ref minus-wheel (- transposed-node-val 12)))
                                   (else (laml-error "tranpose-primary-node: Problems (2)")))))
        transposed-node)))

; ---------------------------------------------------------------------------------------------------------------

; The number of half steps of transposition. A number between -11 and 11.
(define transposition 0)

; Handle a single chord, alone or surrounding phrase. The chord abstraction is also given as parameter.
; phrase is a pharase string
; chord is a symbol or string, as in the chord wheel.
; abstractin is any string to follow the chord, of #f if no abstractions apply.
(define (phrase-with-chord phrase chord abstraction)
  (let* ((transposed-chord (transpose-primary-node (as-symbol chord) transposition))
         (transposed-chord-with-abstraction (chord-presentation transposed-chord abstraction))
         (phrase-length (if (string? phrase) (string-length phrase) 0))
         (abstracted-chord-length 
             (+ (string-length (as-string (as-string transposed-chord)))
                (if abstraction (string-length (as-string abstraction)) 0)))
         (padding-constant (if compact-lines-wanted 0 1))  ; Better in IE: non-compact lines
         (padding (if (> abstracted-chord-length phrase-length)
                      1
                      (- (max (- abstracted-chord-length padding-constant) 0))))     
         (padding-str (string-append (as-string padding) "ex"))
         (phrase-str (string-append "-" (as-string phrase-length) "ex")))
    (if do-show-chords
        (span phrase
              (span 'css:position "relative" 'css:top "-1.2em" 'css:left phrase-str
                    (b 'css:font-style "normal" 'css:color the-chord-color transposed-chord-with-abstraction))
              (span 'css:margin-left padding-str) )
        phrase)))
        

; The textual chord presentation.
; chord is a symbol, as in the chord wheel
; abstraction is an arbitrary string, or #f if no abstractions apply.
; Return a span clause for the chord with its potential abstraction
(define (chord-presentation chord abstraction)
 (let ((abstraction-1 (if abstraction abstraction "")))
  (cond ((eq? chord 'cs)
            (span "C" _ (char-ref 9839) _ abstraction-1))
        ((eq? chord 'eb)
            (span "E" _ (char-ref 9837) _ abstraction-1))
        ((eq? chord 'fs)
            (span "F" _ (char-ref 9839) _ abstraction-1))
        ((eq? chord 'gs)
            (span "G" _ (char-ref 9839) _ abstraction-1))
        ((eq? chord 'hb)
            (span "H" _ (char-ref 9837)))
        (else (span (upcase-string (as-string chord)) _ abstraction-1 )))))

; ---------------------------------------------------------------------------------------------------
; Secondary auxiliary chords, defined as ad hoc XML-in-LAML abstractions.
; These functions are not strictly necessary, but they are convenient for smooth definition of
; the most common chords with abstractions.

; Macro which is used to define auxiliary chords.
(define-syntax chord-variation   
 (syntax-rules ()
   ((chord-variation main-chord-name variation)
    (xml-in-laml-abstraction
    (lambda (cont attr)
      (let ((abstr (defaulted-get-prop 'x attr #f)))
        (main-chord-name cont 'x (if abstr (string-append variation abstr) variation))))))))

; m - minors
(define cm (chord-variation c "m"))
(define csm (chord-variation cs "m"))
(define dm (chord-variation d "m"))
(define ebm (chord-variation eb "m"))
(define em  (chord-variation e "m"))    ; notice the overlap with XHTML em. XHTML em exists as xthml:em
(define fm (chord-variation f "m"))
(define fsm (chord-variation fs "m"))
(define gm (chord-variation g "m"))
(define gsm (chord-variation gs "m"))
(define am (chord-variation a "m"))
(define hbm (chord-variation hb "m"))
(define hm (chord-variation h "m"))

; 7
(define c7 (chord-variation c "7"))
(define cs7 (chord-variation cs "7"))
(define d7 (chord-variation d "7"))
(define eb7 (chord-variation eb "7"))
(define e7  (chord-variation e "7"))  
(define f7 (chord-variation f "7"))
(define fs7 (chord-variation fs "7"))
(define g7 (chord-variation g "7"))
(define gs7 (chord-variation gs "7"))
(define a7 (chord-variation a "7"))
(define hb7 (chord-variation hb "7"))
(define h7 (chord-variation h "7"))

; 6
(define c6 (chord-variation c "6"))
(define cs6 (chord-variation cs "6"))
(define d6 (chord-variation d "6"))
(define eb6 (chord-variation eb "6"))
(define e6  (chord-variation e "6"))  
(define f6 (chord-variation f "6"))
(define fs6 (chord-variation fs "6"))
(define g6 (chord-variation g "6"))
(define gs6 (chord-variation gs "6"))
(define a6 (chord-variation a "6"))
(define hb6 (chord-variation hb "6"))
(define h6 (chord-variation h "6"))

; m7
(define cm7 (chord-variation c "m7"))
(define csm7 (chord-variation cs "m7"))
(define dm7 (chord-variation d "m7"))
(define ebm7 (chord-variation eb "m7"))
(define em7  (chord-variation e "m7"))  
(define fm7 (chord-variation f "m7"))
(define fsm7 (chord-variation fs "m7"))
(define gm7 (chord-variation g "m7"))
(define gsm7 (chord-variation gs "m7"))
(define am7 (chord-variation a "m7"))
(define hbm7 (chord-variation hb "m7"))
(define hm7 (chord-variation h "m7"))




 
  
